@@ -24,7 +24,9 @@ using namespace ipmsg;
  */
 HostList::HostList()
 {
+	IPMSG_FUNC_ENTER( "HostList::HostList()" );
 	IpMsgMutexInit( "HostList::HostList()", &hostListMutex, NULL );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -36,10 +38,12 @@ HostList::HostList()
  */
 HostList::HostList( const HostList& other )
 {
+	IPMSG_FUNC_ENTER( "HostList::HostList( const HostList& other )" );
 	IpMsgMutexInit( "HostList::HostList(HostList&)", &hostListMutex, NULL );
 	Lock( "HostList::HostList(HostList&)" );
 	CopyFrom( other );
 	Unlock( "HostList::HostList(HostList&)" );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -50,7 +54,9 @@ HostList::HostList( const HostList& other )
  */
 HostList::~HostList()
 {
+	IPMSG_FUNC_ENTER( "HostList::~HostList()" );
 	IpMsgMutexDestroy( "HostList::~HostList()", &hostListMutex );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -64,11 +70,12 @@ HostList::~HostList()
 HostList&
 HostList::operator=( const HostList& other )
 {
+	IPMSG_FUNC_ENTER( "HostList& HostList::operator=( const HostList& other )" );
 	IpMsgMutexInit( "HostList::operator=(HostList&)", &hostListMutex, NULL );
 	Lock( "HostList::operator=(HostList&)" );
 	CopyFrom( other );
 	Unlock( "HostList::operator=(HostList&)" );
-	return *this;
+	IPMSG_FUNC_RETURN( *this );
 }
 
 /**
@@ -78,7 +85,9 @@ HostList::operator=( const HostList& other )
 void
 HostList::CopyFrom( const HostList& other )
 {
+	IPMSG_FUNC_ENTER( "void HostList::CopyFrom( const HostList& other )" );
 	items = other.items;
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -88,7 +97,9 @@ HostList::CopyFrom( const HostList& other )
 void
 HostList::Lock( const char *pos ) const
 {
+	IPMSG_FUNC_ENTER( "void HostList::Lock( const char *pos ) const" );
 	IpMsgMutexLock( pos, const_cast< pthread_mutex_t* >( &hostListMutex ) );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -98,7 +109,9 @@ HostList::Lock( const char *pos ) const
 void
 HostList::Unlock( const char *pos ) const
 {
+	IPMSG_FUNC_ENTER( "void HostList::Unlock( const char *pos ) const" );
 	IpMsgMutexUnlock( pos, const_cast< pthread_mutex_t * >( &hostListMutex ) );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -108,7 +121,8 @@ HostList::Unlock( const char *pos ) const
 std::vector<HostListItem>::iterator
 HostList::begin()
 {
-	return items.begin();
+	IPMSG_FUNC_ENTER( "std::vector<HostListItem>::iterator HostList::begin()" );
+	IPMSG_FUNC_RETURN( items.begin() );
 }
 
 /**
@@ -118,7 +132,8 @@ HostList::begin()
 std::vector<HostListItem>::iterator
 HostList::end()
 {
-	return items.end();
+	IPMSG_FUNC_ENTER( "std::vector<HostListItem>::iterator HostList::end()" );
+	IPMSG_FUNC_RETURN(  items.end() );
 }
 
 /**
@@ -128,10 +143,11 @@ HostList::end()
 int
 HostList::size() const
 {
+	IPMSG_FUNC_ENTER( "int HostList::size() const" );
 	Lock( "HostList::size()" );
 	int ret = items.size();
 	Unlock( "HostList::size()" );
-	return ret;
+	IPMSG_FUNC_RETURN( ret );
 }
 
 /**
@@ -140,9 +156,11 @@ HostList::size() const
 void
 HostList::clear()
 {
+	IPMSG_FUNC_ENTER( "void HostList::clear()" );
 	Lock( "HostList::clear()" );
 	items.clear();
 	Unlock( "HostList::clear()" );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -151,8 +169,10 @@ HostList::clear()
 void
 HostListItem::QueryVersionInfo()
 {
+	IPMSG_FUNC_ENTER( "void HostListItem::QueryVersionInfo()" );
 	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
 	agent->QueryVersionInfo( *this );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -161,8 +181,10 @@ HostListItem::QueryVersionInfo()
 void
 HostListItem::QueryAbsenceInfo()
 {
+	IPMSG_FUNC_ENTER( "void HostListItem::QueryAbsenceInfo()" );
 	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
 	agent->QueryAbsenceInfo( *this );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -173,6 +195,7 @@ HostListItem::QueryAbsenceInfo()
 bool
 HostListItem::IsLocalHost() const
 {
+	IPMSG_FUNC_ENTER( "bool HostListItem::IsLocalHost() const" );
 	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
 	std::vector<NetworkInterface> nics = agent->NICs;
 	for( unsigned int i = 0; i < nics.size(); i++ ){
@@ -180,75 +203,129 @@ HostListItem::IsLocalHost() const
 #if defined(INFO) || !defined(NDEBUG)
 			printf("LOCALHOST\n");fflush(stdout);
 #endif
-			return true;
+			IPMSG_FUNC_RETURN( true );
 		}
 	}
 #if defined(INFO) || !defined(NDEBUG)
 	printf("OTHERHOST\n");fflush(stdout);
 #endif
-	return false;
+	IPMSG_FUNC_RETURN( false );
 }
 
 /**
  * ۥȾۥȥꥹȤɲä롣
  * @param host ۥȾ
+ * @retval Ͽ
  */
-void
-HostList::AddHost( const HostListItem& host )
+int
+HostList::AddHost( const HostListItem& host, bool isPermitSameHardwareAddress )
 {
+	IPMSG_FUNC_ENTER( "void HostList::AddHost( const HostListItem& host, bool isPermitSameHardwareAddress )" );
 	Lock( "HostList::AddHost()" );
 	bool is_found = false;
+//	isPermitSameHardwareAddress = true;
 
 	IpMessengerAgentImpl *agent = IpMessengerAgentImpl::GetInstance();
 	std::string localhostName = agent->HostName();
 	std::vector<NetworkInterface> nics = agent->NICs;
-	for( unsigned int i = 1; i < nics.size(); i++ ){
+	//ƬΥåȥɥ쥹եߥNICõץ饤ޥ(ɥ쥹)Ȥư١θõΥǥå롣
+	int nicStartIndex = 1;
+	if ( !agent->haveIPv4Nic && !agent->haveIPv6Nic ) {
+			IPMSG_FUNC_RETURN( 0 );
+	} else if ( agent->haveIPv4Nic && agent->haveIPv6Nic && host.AddressFamily() == AF_INET6 ) {
+		//IPv4,IPv6ξȤʤIPv6ͥ衣
+		for( unsigned int i = 0; i < nics.size(); i++ ){
+			if ( nics[i].AddressFamily() == AF_INET6 ) {
+				nicStartIndex = i + 1;
+				break;
+			}
+		}
+	} else {
+		nicStartIndex = 1;
+	}
+	//õǥå֤饹
+	for( unsigned int i = nicStartIndex; i < nics.size(); i++ ){
 #if defined(INFO) || !defined(NDEBUG)
 		printf("AddHost HOST CHECK IpAddress=%s addr=%s\n", host.IpAddress().c_str(), nics[i].IpAddress().c_str() );fflush(stdout);
 #endif
+
 		if ( host.IpAddress() == nics[i].IpAddress() ) {
+#if defined(INFO) || !defined(NDEBUG)
+			printf("AddHost HOST MATCH\n" );fflush(stdout);
+#endif
 			Unlock( "HostList::AddHost()" );
-			return;
+			IPMSG_FUNC_RETURN( 0 );
 		}
 	}
+	//IPɥ쥹NICΥ֥ɥ㥹ȡͥåȥΥɥ쥹Ȱפ̵롣ʤ֤󤢤ꤨʤɡ
 	for( unsigned int i = 0; i < nics.size(); i++ ){
 #if defined(INFO) || !defined(NDEBUG)
-		printf("AddHost HOST CHECK IpAddress=%s addr=%s\n", host.IpAddress().c_str(), nics[i].IpAddress().c_str() );fflush(stdout);
+		printf("AddHost ADDR CHECK IpAddress=%s net=%s broad=%s\n", host.IpAddress().c_str(), nics[i].NetworkAddress().c_str(), nics[i].BroadcastAddress().c_str() );fflush(stdout);
 #endif
 		if ( host.IpAddress() == nics[i].NetworkAddress() ||
 			 host.IpAddress() == nics[i].BroadcastAddress() ){
+#if defined(INFO) || !defined(NDEBUG)
+			printf("AddHost ADDR MATCH\n" );fflush(stdout);
+#endif
 			Unlock( "HostList::AddHost()" );
-			return;
+			IPMSG_FUNC_RETURN( 0 );
 		}
 	}
 #if defined(INFO) || !defined(NDEBUG)
 	printf("AddHost HOST CHECK IpAddress=%s addr=%s\n", host.IpAddress().c_str(), nics[0].IpAddress().c_str() );fflush(stdout);
 	printf("AddHost HOST CHECK HostName=%s localhost=%s\n", host.HostName().c_str(), localhostName.c_str() );fflush(stdout);
 #endif
-	if ( host.IpAddress() == "127.0.0.1" ){
+	//IPɥ쥹롼ץХåɥ쥹Ȱפ̵롣
+	if ( host.IpAddress() == "127.0.0.1" || host.IpAddress() == "::1" ){
 #if defined(INFO) || !defined(NDEBUG)
 		printf("IGNORE HOST.Because host IpAddress local loopback.\n" );fflush(stdout);
 #endif
 		Unlock( "HostList::AddHost()" );
-		return;
+		IPMSG_FUNC_RETURN( 0 );
 	}
+	//IPɥ쥹NICIPɥ쥹ȰפΤ˥ۥ̾Ȱפʤ̵롣
 	if ( host.IpAddress() == nics[0].IpAddress() && host.HostName() != localhostName ){
+#if defined(INFO) || !defined(NDEBUG)
+		printf("MATCH IPADDRESS but not same hostname.\n" );fflush(stdout);
+#endif
 		Unlock( "HostList::AddHost()" );
-		return;
+		IPMSG_FUNC_RETURN( 0 );
 	}
-	for( unsigned int i = 0; i < items.size(); i++ ){
-		HostListItem tmpHost = items.at( i );
-		if ( tmpHost.Equals( host ) ) {
-			is_found = true;
-			break;
+	std::vector<HostListItem>::iterator tmpHost;
+	for( tmpHost = items.begin(); tmpHost != items.end(); tmpHost++ ){
+		if ( isPermitSameHardwareAddress ) {
+			if ( tmpHost->Equals( host ) ) {
+				is_found = true;
+				break;
+			}
+		} else {
+#if defined(INFO) || !defined(NDEBUG)
+		printf("SEARCHING HARDWARE ADDRESS...base[%s] check[%s]\n", host.HardwareAddress().c_str(), tmpHost->HardwareAddress().c_str() );fflush(stdout);
+#endif
+			if ( tmpHost->EqualsHardwareAddress( host ) ) {
+#if defined(INFO) || !defined(NDEBUG)
+		printf("FOUND ADDRESS %s\n", host.HardwareAddress().c_str() );fflush(stdout);
+#endif
+				is_found = true;
+				//IPv6IPv4ͥ褹롣
+				if ( host.AddressFamily() == AF_INET6 && tmpHost->AddressFamily() == AF_INET ) {
+					*tmpHost = host;
+#if defined(INFO) || !defined(NDEBUG)
+		printf("ͥ褵ޤ%s\n", host.HardwareAddress().c_str() );fflush(stdout);
+#endif
+				}
+				break;
+			}
 		}
 	}
+	int ret = 0;
 	if ( !is_found ) {
 #if defined(INFO) || !defined(NDEBUG)
 		printf("AddHost Nickname=%s\n", host.Nickname().c_str() );fflush(stdout);
 		printf("AddHost GroupName=%s\n", host.GroupName().c_str() );fflush(stdout);
 #endif
 		items.push_back( host );
+		ret = 1;
 	}
 	if ( agent->GetSortHostListComparator() != NULL ){
 		if ( items.size() > 0 ) {
@@ -256,6 +333,7 @@ HostList::AddHost( const HostListItem& host )
 		}
 	}
 	Unlock( "HostList::AddHost()" );
+	IPMSG_FUNC_RETURN( ret );
 }
 
 /**
@@ -265,9 +343,11 @@ HostList::AddHost( const HostListItem& host )
 void
 HostList::Delete( std::vector<HostListItem>::iterator &it )
 {
+	IPMSG_FUNC_ENTER( "void HostList::Delete( std::vector<HostListItem>::iterator &it )" );
 	Lock( "HostList::Delete()" );
 	items.erase( it );
 	Unlock( "HostList::Delete()" );
+	IPMSG_FUNC_EXIT;
 }
 /**
  * ۥȾۥȥꥹȤ롣
@@ -276,14 +356,24 @@ HostList::Delete( std::vector<HostListItem>::iterator &it )
 void
 HostList::DeleteHostByAddress( std::string addr )
 {
+	IPMSG_FUNC_ENTER( "void HostList::DeleteHostByAddress( std::string addr )" );
 	Lock( "HostList::DeleteHostIpAddress()" );
+	struct sockaddr_storage ss;
+	if ( createSockAddrIn( &ss, addr, 0 ) == NULL ){
+		IPMSG_FUNC_EXIT;
+	}
 	for( std::vector<HostListItem>::iterator ix = items.begin(); ix < items.end(); ix++ ){
-		if ( ix->IpAddress() == addr ) {
+		struct sockaddr_storage ixss;
+		if ( createSockAddrIn( &ixss, ix->IpAddress(), 0 ) == NULL ){
+			IPMSG_FUNC_EXIT;
+		}
+		if ( isSameSockAddrIn( ss, ixss ) ){
 			items.erase( ix );
 			break;
 		}
 	}
 	Unlock( "HostList::DeleteHostByAddress()" );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -293,8 +383,9 @@ HostList::DeleteHostByAddress( std::string addr )
  * @retval ۥȥꥹʸ
  */
 std::string
-HostList::ToString( int start, const struct sockaddr_in *addr )
+HostList::ToString( int start, const struct sockaddr_storage *addr )
 {
+	IPMSG_FUNC_ENTER( "std::string HostList::ToString( int start, const struct sockaddr_storage *addr )" );
 	Lock( "HostList::ToString" );
 	char buf[MAX_UDPBUF];
 	std::string ret;
@@ -312,7 +403,7 @@ HostList::ToString( int start, const struct sockaddr_in *addr )
 			std::vector<NetworkInterface> nics = agent->NICs;
 			std::string localaddr = nics[0].IpAddress();
 			for( unsigned int i = 0; i < nics.size(); i++ ){
-				if ( isSameNetwork( addr->sin_addr, nics[i].NativeNetworkAddress(), nics[i].NativeNetMask() ) ){
+				if ( isSameNetwork( addr, nics[i].NetworkAddress(), nics[i].NetMask() ) ){
 					localaddr = nics[i].IpAddress();
 					break;
 				}
@@ -326,6 +417,9 @@ HostList::ToString( int start, const struct sockaddr_in *addr )
 							item.Nickname() == "" ? "\b" : item.Nickname().c_str(),
 							item.GroupName() == "" ? "\b" : item.GroupName().c_str() );
 		} else {
+			if ( item.AddressFamily() != addr->ss_family ) {
+				continue;
+			}
 			len = snprintf( buf, sizeof( buf ), "%s\a%s\a%ld\a%s\a%s\a%s\a%s\a",
 							item.UserName() == "" ? "\b" : item.UserName().c_str(),
 							item.HostName() == "" ? "\b" : item.HostName().c_str(),
@@ -346,8 +440,11 @@ HostList::ToString( int start, const struct sockaddr_in *addr )
 	}
 	snprintf( buf, sizeof( buf ), "%-5d\a%-5d\a", start , hostCount );
 	ret = buf + ret;
+#if defined(INFO) || !defined(NDEBUG)
+	printf( "HostList::ToString [%s]\n", ret.c_str() );fflush(stdout);
+#endif
 	Unlock( "HostList::ToString" );
-	return ret;
+	IPMSG_FUNC_RETURN( ret );
 }
 
 /**
@@ -358,16 +455,16 @@ HostList::ToString( int start, const struct sockaddr_in *addr )
 HostListItem
 HostList::CreateHostListItemFromPacket( const Packet& packet )
 {
+	IPMSG_FUNC_ENTER( "HostListItem HostList::CreateHostListItemFromPacket( const Packet& packet )" );
 	HostListItem ret;
 	ret.setHostName( packet.HostName() );
 	ret.setUserName( packet.UserName() );
 	ret.setCommandNo( packet.CommandMode() | packet.CommandOption() );
-	char tmp[100];
-	ret.setIpAddress( inet_ntop( AF_INET, &packet.Addr().sin_addr, tmp, sizeof( tmp ) ) );
+	ret.setIpAddress( getSockAddrInRawAddress( packet.Addr() ) );
 #if defined(INFO) || !defined(NDEBUG)
-	printf( "CreateHostListItemFromPacket port %d\n", ntohs( packet.Addr().sin_port ) );fflush(stdout);
+	printf( "CreateHostListItemFromPacket port %d\n", ntohs( getSockAddrInPortNo( packet.Addr() ) ) );fflush(stdout);
 #endif
-	ret.setPortNo( ntohs( packet.Addr().sin_port ) );
+	ret.setPortNo( ntohs( getSockAddrInPortNo( packet.Addr() ) ) );
 	unsigned int loc = packet.Option().find_first_of( '\0' );
 	if ( loc == std::string::npos ) {
 		ret.setNickname( packet.Option() );
@@ -376,7 +473,7 @@ HostList::CreateHostListItemFromPacket( const Packet& packet )
 		ret.setNickname( packet.Option().substr( 0, loc ) );
 		ret.setGroupName( packet.Option().substr( loc + 1 ) );
 	}
-	return ret;
+	IPMSG_FUNC_RETURN( ret );
 }
 
 /**
@@ -385,18 +482,37 @@ HostList::CreateHostListItemFromPacket( const Packet& packet )
  * @retval HostListItem
  */
 std::vector<HostListItem>::iterator
-HostList::FindHostByHostName( std::string hostName )
+HostList::FindHostByHostName( std::string hostName, int addressFamily )
 {
+	IPMSG_FUNC_ENTER( "std::vector<HostListItem>::iterator HostList::FindHostByHostName( std::string hostName )" );
 	Lock( "HostList::FindHostByHostName()" );
 	std::vector<HostListItem>::iterator ret = end();
+//	printf( "HostName [%s]\n", hostName.c_str() );fflush(stdout);
+//	printf( "HostList.size [%d]\n", items.size() );fflush(stdout);
+
+#if defined(DEBUG)
+	bool isMatch = false;
+#endif
 	for( std::vector<HostListItem>::iterator ix = begin(); ix < end(); ix++ ){
-		if ( ix->HostName() == hostName ) {
+#if defined(INFO) || !defined(NDEBUG)
+		printf( "ix->HostName [%s]\n", ix->HostName().c_str() );fflush(stdout);
+#endif
+		if ( ix->HostName() == hostName && addressFamily == ix->AddressFamily() ) {
+#if defined(DEBUG)
+			printf("HOSTNAME MATCH!!!\n");fflush(stdout);
+			isMatch = true;
+#endif
 			ret = ix;
 			break;
 		}
 	}
+#if defined(DEBUG)
+	if ( !isMatch ){
+		printf("HOSTNAME UNMATCH!!!\n");fflush(stdout);
+	}
+#endif
 	Unlock( "HostList::FindHostByHostName()" );
-	return ret;
+	IPMSG_FUNC_RETURN( ret );
 }
 
 /**
@@ -407,13 +523,22 @@ HostList::FindHostByHostName( std::string hostName )
 std::vector<HostListItem>::iterator
 HostList::FindHostByAddress( std::string addr )
 {
+	IPMSG_FUNC_ENTER( "std::vector<HostListItem>::iterator HostList::FindHostByAddress( std::string addr )" );
 	Lock( "HostList::FindHostByAddress()" );
 	std::vector<HostListItem>::iterator ret = end();
+	struct sockaddr_storage ss;
+	if ( createSockAddrIn( &ss, addr, 0 ) == NULL ){
+		IPMSG_FUNC_RETURN( ret );
+	}
 	for( std::vector<HostListItem>::iterator ix = begin(); ix < end(); ix++ ){
 #if defined(DEBUG)
 		printf("HOST CHECK IpAddress=%s addr=%s\n", ix->IpAddress().c_str(), addr.c_str() );fflush(stdout);
 #endif
-		if ( ix->IpAddress() == addr ) {
+		struct sockaddr_storage ixss;
+		if ( createSockAddrIn( &ixss, ix->IpAddress(), 0 ) == NULL ){
+			IPMSG_FUNC_RETURN( ret );
+		}
+		if ( isSameSockAddrIn( ss, ixss ) ){
 #if defined(DEBUG)
 			printf("HOST FOUND!!!\n");fflush(stdout);
 #endif
@@ -425,7 +550,34 @@ HostList::FindHostByAddress( std::string addr )
 	printf("HOST NOT FOUND!!!\n");fflush(stdout);
 #endif
 	Unlock( "HostList::FindHostByAddress()" );
-	return ret;
+	IPMSG_FUNC_RETURN( ret );
+}
+
+/**
+ * IPɥ쥹Υå
+ * @param IPɥ쥹
+ */
+void
+HostListItem::setIpAddress( const std::string val )
+{
+	IPMSG_FUNC_ENTER( "void HostListItem::setIpAddress( const std::strin val )" );
+	_IpAddress = val;
+	sockaddr_storage ss;
+	createSockAddrIn( &ss, val, 0 );
+	_AddressFamily = ss.ss_family;
+	_HardwareAddress = convertIpAddressToMacAddress( val, IpMessengerAgentImpl::GetInstance()->NICs );
+	IPMSG_FUNC_EXIT;
+}
+
+/**
+ * IPɥ쥹Υå
+ * @retval IPɥ쥹
+ */
+std::string
+HostListItem::IpAddress() const
+{
+	IPMSG_FUNC_ENTER( "std::string HostListItem::IpAddress() const" );
+	IPMSG_FUNC_RETURN( _IpAddress );
 }
 
 /**
@@ -436,7 +588,8 @@ HostList::FindHostByAddress( std::string addr )
 bool
 HostListItem::IsFileAttachSupport() const
 {
-	return CommandNo() & IPMSG_FILEATTACHOPT;
+	IPMSG_FUNC_ENTER( "bool HostListItem::IsFileAttachSupport() const" );
+	IPMSG_FUNC_RETURN( CommandNo() & IPMSG_FILEATTACHOPT );
 }
 
 /**
@@ -447,7 +600,8 @@ HostListItem::IsFileAttachSupport() const
 bool
 HostListItem::IsEncryptSupport() const
 {
-	return CommandNo() & IPMSG_ENCRYPTOPT;
+	IPMSG_FUNC_ENTER( "bool HostListItem::IsEncryptSupport() const" );
+	IPMSG_FUNC_RETURN( CommandNo() & IPMSG_ENCRYPTOPT );
 }
 
 /**
@@ -458,7 +612,8 @@ HostListItem::IsEncryptSupport() const
 bool
 HostListItem::IsAbsence() const
 {
-	return CommandNo() & IPMSG_ABSENCEOPT;
+	IPMSG_FUNC_ENTER( "bool HostListItem::IsAbsence() const" );
+	IPMSG_FUNC_RETURN( CommandNo() & IPMSG_ABSENCEOPT );
 }
 
 /**
@@ -470,8 +625,23 @@ HostListItem::IsAbsence() const
 bool
 HostListItem::Equals( const HostListItem& item ) const
 {
-	return	Compare( item ) == 0;
+	IPMSG_FUNC_ENTER( "bool HostListItem::Equals( const HostListItem& item ) const" );
+	IPMSG_FUNC_RETURN( Compare( item ) == 0 );
 }
+
+/**
+ * ۥȥꥹȥƥ४֥ȤΥϡɥɥ쥹ʬȰפ뤫֤
+ * @param item ۥȥꥹȥƥ
+ * @retval true:
+ * @retval false:פʤ
+ */
+bool
+HostListItem::EqualsHardwareAddress( const HostListItem& item ) const
+{
+	IPMSG_FUNC_ENTER( "bool HostListItem::EqualsHardwareAddress( const HostListItem& item ) const" );
+	IPMSG_FUNC_RETURN( CompareHardwareAddress( item ) == 0 );
+}
+
 /**
  * ӡ
  * @param item ۥȾ1
@@ -482,29 +652,38 @@ HostListItem::Equals( const HostListItem& item ) const
 int
 HostListItem::Compare( const HostListItem& item ) const
 {
-//	if ( item.UserName()  == UserName() &&
-//		 item.HostName()  == HostName() &&
-//		 item.IpAddress() == IpAddress() &&
-//		 item.Nickname()  == Nickname() &&
-//		 item.GroupName() == GroupName() &&
-//		 item.PortNo()    == PortNo() {
+	IPMSG_FUNC_ENTER( "int HostListItem::Compare( const HostListItem& item ) const" );
 	if ( item.UserName()  == UserName() &&
 		 item.HostName()  == HostName() &&
 		 item.IpAddress() == IpAddress() ){
-		return 0;
+		IPMSG_FUNC_RETURN( 0 );
 	}
-//	if ( item.UserName()  > UserName() &&
-//		 item.HostName()  > HostName() &&
-//		 item.IpAddress() > IpAddress() &&
-//		 item.Nickname()  > Nickname() &&
-//		 item.GroupName() > GroupName() &&
-//		 item.PortNo()    > PortNo() {
 	if ( item.UserName()  > UserName() &&
 		 item.HostName()  > HostName() &&
 		 item.IpAddress() > IpAddress() ){
-		return 1;
+		IPMSG_FUNC_RETURN( 1 );
 	}
-	return -1;
+	IPMSG_FUNC_RETURN(  -1 );
+}
+
+/**
+ * ϡɥɥ쥹Ǥӡ
+ * @param item ۥȾ1
+ * @retval -n:*this礭
+ * @retval 0:item*this
+ * @retval +n:item礭
+ */
+int
+HostListItem::CompareHardwareAddress( const HostListItem& item ) const
+{
+	IPMSG_FUNC_ENTER( "int HostListItem::CompareHardwareAddress( const HostListItem& item ) const" );
+	if ( item.HardwareAddress() == HardwareAddress() ){
+		IPMSG_FUNC_RETURN( 0 );
+	}
+	if ( item.HardwareAddress() > HardwareAddress() ){
+		IPMSG_FUNC_RETURN( 1 );
+	}
+	IPMSG_FUNC_RETURN(  -1 );
 }
 
 /**
@@ -514,6 +693,7 @@ HostListItem::Compare( const HostListItem& item ) const
 void
 HostList::qsort( HostListComparator *comparator, int left, int right )
 {
+	IPMSG_FUNC_ENTER( "void HostList::qsort( HostListComparator *comparator, int left, int right )" );
 	//ϰϤγϡλ
 	int i = left, j = right;
 #if defined(INFO) || !defined(NDEBUG)
@@ -545,6 +725,7 @@ HostList::qsort( HostListComparator *comparator, int left, int right )
 	if ( j + 1 < right ) {	//ͤαˣİʾǤб򥽡Ȥ롣
 		qsort( comparator, j + 1, right );
 	}
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -554,9 +735,11 @@ HostList::qsort( HostListComparator *comparator, int left, int right )
 void
 HostList::sort( HostListComparator *comparator )
 {
+	IPMSG_FUNC_ENTER( "void HostList::sort( HostListComparator *comparator )" );
 	if ( items.size() > 0 ) {
 		qsort( comparator, 0, items.size() - 1 );
 	}
+	IPMSG_FUNC_EXIT;
 }
 
 class IpMsgGetGroupListComparator: public HostListComparator{
@@ -592,6 +775,7 @@ class IpMsgGetGroupListComparator: public HostListComparator{
 std::vector<GroupItem>
 HostList::GetGroupList()
 {
+	IPMSG_FUNC_ENTER( "std::vector<GroupItem> HostList::GetGroupList()" );
 	std::vector<GroupItem> ret;
 	HostList tmp = *this;
 	tmp.sort( new IpMsgGetGroupListComparator() );
@@ -606,7 +790,7 @@ HostList::GetGroupList()
 		hostName = ixhost->HostName();
 		encodingName = ixhost->EncodingName();
 	}
-	return ret;
+	IPMSG_FUNC_RETURN( ret );
 }
 
 //end of source
