@@ -20,7 +20,11 @@ static IpMessengerAgent *myInstance = NULL;
 static pthread_mutex_t instanceMutex;
 static int mutex_init_result = IpMsgMutexInit( "IpMessenger::Global", &instanceMutex, NULL );
 
-IpMessengerEvent::~IpMessengerEvent(){};
+IpMessengerEvent::~IpMessengerEvent()
+{
+	IPMSG_FUNC_ENTER("IpMessengerEvent::~IpMessengerEvent()");
+	IPMSG_FUNC_EXIT;
+};
 /**
  * IP å󥸥㥨ȥ饹Υ󥹥󥹤롣
  * <ul>
@@ -30,12 +34,14 @@ IpMessengerEvent::~IpMessengerEvent(){};
 IpMessengerAgent *
 IpMessengerAgent::GetInstance()
 {
+	IPMSG_FUNC_ENTER("IpMessengerAgent * IpMessengerAgent::GetInstance()");
+	mutex_init_result = 0; //fix warnings. but no effect.
 	IpMsgMutexLock( "IpMessengerAgent::GetInstance()", &instanceMutex );
 	if ( myInstance == NULL ) {
 		myInstance = new IpMessengerAgent();
 	}
 	IpMsgMutexUnlock( "IpMessengerAgent::GetInstance()", &instanceMutex );
-	return myInstance;
+	IPMSG_FUNC_RETURN( myInstance );
 }
 
 /**
@@ -48,14 +54,16 @@ IpMessengerAgent::GetInstance()
 void
 IpMessengerAgent::Release()
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::Release()");
 	IpMsgMutexLock( "IpMessengerAgent::Release()", &instanceMutex );
 	if ( myInstance == NULL ) {
 		IpMsgMutexUnlock( "IpMessengerAgent::Release()", &instanceMutex );
-		return;
+		IPMSG_FUNC_EXIT;
 	}
 	delete myInstance;
 	myInstance = NULL;
 	IpMsgMutexUnlock( "IpMessengerAgent::Release()", &instanceMutex );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -63,7 +71,19 @@ IpMessengerAgent::Release()
  */
 IpMessengerAgent::IpMessengerAgent()
 {
+	IPMSG_FUNC_ENTER("IpMessengerAgent::IpMessengerAgent()");
+	if ( isSupportIPv4() ) {
+		printf("This host support IPv4.\n");
+	} else {
+		printf("This host not support IPv4.\n");
+	}
+	if ( isSupportIPv6() ) {
+		printf("This host support IPv6.\n");
+	} else {
+		printf("This host not support IPv6.\n");
+	}
 	ipmsgImpl = IpMessengerAgentImpl::GetInstance();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -71,7 +91,9 @@ IpMessengerAgent::IpMessengerAgent()
  */
 IpMessengerAgent::~IpMessengerAgent()
 {
+	IPMSG_FUNC_ENTER("IpMessengerAgent::~IpMessengerAgent()");
 	IpMessengerAgentImpl::Release();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -83,7 +105,9 @@ IpMessengerAgent::~IpMessengerAgent()
 void
 IpMessengerAgent::StartNetwork()
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::StartNetwork()");
 	ipmsgImpl->StartNetwork();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -93,7 +117,9 @@ IpMessengerAgent::StartNetwork()
 void
 IpMessengerAgent::StartNetwork( const std::vector<NetworkInterface>& nics )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::StartNetwork( const std::vector<NetworkInterface>& nics )");
 	ipmsgImpl->StartNetwork( nics );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -102,7 +128,9 @@ IpMessengerAgent::StartNetwork( const std::vector<NetworkInterface>& nics )
 void
 IpMessengerAgent::StopNetwork()
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::StopNetwork()");
 	ipmsgImpl->StopNetwork();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -114,7 +142,9 @@ IpMessengerAgent::StopNetwork()
 void
 IpMessengerAgent::RestartNetwork()
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::RestartNetwork()");
 	ipmsgImpl->RestartNetwork();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -124,7 +154,9 @@ IpMessengerAgent::RestartNetwork()
 void
 IpMessengerAgent::RestartNetwork( const std::vector<NetworkInterface>& nics )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::RestartNetwork( const std::vector<NetworkInterface>& nics )");
 	ipmsgImpl->RestartNetwork( nics );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -134,7 +166,8 @@ IpMessengerAgent::RestartNetwork( const std::vector<NetworkInterface>& nics )
 FileNameConverter *
 IpMessengerAgent::GetFileNameConverter() const
 {
-	return ipmsgImpl->GetFileNameConverter();
+	IPMSG_FUNC_ENTER("FileNameConverter *IpMessengerAgent::GetFileNameConverter() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetFileNameConverter() );
 }
 
 /**
@@ -144,7 +177,9 @@ IpMessengerAgent::GetFileNameConverter() const
 void
 IpMessengerAgent::SetFileNameConverter( const FileNameConverter *conv )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::SetFileNameConverter( const FileNameConverter *conv )");
 	ipmsgImpl->SetFileNameConverter( conv );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -154,7 +189,8 @@ IpMessengerAgent::SetFileNameConverter( const FileNameConverter *conv )
 HostListComparator *
 IpMessengerAgent::GetSortHostListComparator() const
 {
-	return ipmsgImpl->GetSortHostListComparator();
+	IPMSG_FUNC_ENTER("HostListComparator *IpMessengerAgent::GetSortHostListComparator() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetSortHostListComparator() );
 }; 
 
 /**
@@ -164,7 +200,9 @@ IpMessengerAgent::GetSortHostListComparator() const
 void
 IpMessengerAgent::SetSortHostListComparator( const HostListComparator *comparator )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::SetSortHostListComparator( const HostListComparator *comparator )");
 	ipmsgImpl->SetSortHostListComparator( comparator );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -174,7 +212,8 @@ IpMessengerAgent::SetSortHostListComparator( const HostListComparator *comparato
 IpMessengerEvent *
 IpMessengerAgent::GetEventObject() const
 {
-	return ipmsgImpl->GetEventObject();
+	IPMSG_FUNC_ENTER("IpMessengerEvent *IpMessengerAgent::GetEventObject() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetEventObject() );
 }; 
 
 /**
@@ -184,7 +223,9 @@ IpMessengerAgent::GetEventObject() const
 void
 IpMessengerAgent::SetEventObject( const IpMessengerEvent *evt )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::SetEventObject( const IpMessengerEvent *evt )");
 	ipmsgImpl->SetEventObject( evt );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -192,9 +233,11 @@ IpMessengerAgent::SetEventObject( const IpMessengerEvent *evt )
  * @param nics ͥåȥ󥿡եΰ
  */
 void
-IpMessengerAgent::GetNetworkInterfaceInfo( std::vector<NetworkInterface>& nics )
+IpMessengerAgent::GetNetworkInterfaceInfo( std::vector<NetworkInterface>& nics, bool useIPv6 )
 {
-	IpMessengerAgentImpl::GetNetworkInterfaceInfo( nics );
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::GetNetworkInterfaceInfo( std::vector<NetworkInterface>& nics, bool useIPv6 )");
+	IpMessengerAgentImpl::GetNetworkInterfaceInfo( nics, useIPv6 );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -203,7 +246,9 @@ IpMessengerAgent::GetNetworkInterfaceInfo( std::vector<NetworkInterface>& nics )
 void
 IpMessengerAgent::Login( std::string nickname, std::string groupName )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::Login( std::string nickname, std::string groupName )");
 	ipmsgImpl->Login( nickname, groupName );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -212,7 +257,9 @@ IpMessengerAgent::Login( std::string nickname, std::string groupName )
 void
 IpMessengerAgent::Logout()
 {	
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::Logout()");
 	ipmsgImpl->Logout();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -222,7 +269,8 @@ IpMessengerAgent::Logout()
 HostList&
 IpMessengerAgent::GetHostList()
 {
-	return ipmsgImpl->GetHostList();
+	IPMSG_FUNC_ENTER("HostList& IpMessengerAgent::GetHostList()");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetHostList() );
 }
 
 /**
@@ -232,7 +280,8 @@ IpMessengerAgent::GetHostList()
 HostList&
 IpMessengerAgent::UpdateHostList()
 {
-	return ipmsgImpl->UpdateHostList();
+	IPMSG_FUNC_ENTER("HostList& IpMessengerAgent::UpdateHostList()");
+	IPMSG_FUNC_RETURN( ipmsgImpl->UpdateHostList() );
 }
 
 /**
@@ -242,7 +291,8 @@ IpMessengerAgent::UpdateHostList()
 bool
 IpMessengerAgent::IsAbsence() const
 {
-	return ipmsgImpl->IsAbsence();
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::IsAbsence() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->IsAbsence() );
 }
 /**
  * Ժߥ⡼ɤ򥯥ꥢ롣
@@ -250,7 +300,9 @@ IpMessengerAgent::IsAbsence() const
 void
 IpMessengerAgent::ResetAbsence()
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::ResetAbsence()");
 	ipmsgImpl->ResetAbsence();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -261,7 +313,9 @@ IpMessengerAgent::ResetAbsence()
 void
 IpMessengerAgent::SetAbsence( std::string encoding, std::vector<AbsenceMode> absenceModes )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::SetAbsence( std::string encoding, std::vector<AbsenceMode> absenceModes )");
 	ipmsgImpl->SetAbsence( encoding, absenceModes );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -277,7 +331,8 @@ IpMessengerAgent::SetAbsence( std::string encoding, std::vector<AbsenceMode> abs
 bool
 IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, bool isLockPassword, int hostCountAtSameTime, bool IsNoLogging, unsigned long opt )
 {
-	return ipmsgImpl->SendMsg( host, msg, isSecret, isLockPassword, hostCountAtSameTime, IsNoLogging, opt );
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, bool isLockPassword, int hostCountAtSameTime, bool IsNoLogging, unsigned long opt )");
+	IPMSG_FUNC_RETURN( ipmsgImpl->SendMsg( host, msg, isSecret, isLockPassword, hostCountAtSameTime, IsNoLogging, opt ) );
 }
 
 /**
@@ -294,7 +349,8 @@ IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, bo
 bool
 IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, AttachFile& file, bool isLockPassword, int hostCountAtSameTime, bool IsNoLogging, unsigned long opt )
 {
-	return ipmsgImpl->SendMsg( host, msg, isSecret, file, isLockPassword, hostCountAtSameTime, IsNoLogging, opt );
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, AttachFile& file, bool isLockPassword, int hostCountAtSameTime, bool IsNoLogging, unsigned long opt )");
+	IPMSG_FUNC_RETURN( ipmsgImpl->SendMsg( host, msg, isSecret, file, isLockPassword, hostCountAtSameTime, IsNoLogging, opt ) );
 }
 
 /**
@@ -311,7 +367,8 @@ IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, At
 bool
 IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, AttachFileList& files, bool isLockPassword, int hostCountAtSameTime, bool IsNoLogging, unsigned long opt )
 {
-	return ipmsgImpl->SendMsg( host, msg, isSecret, files, isLockPassword, hostCountAtSameTime, IsNoLogging, opt );
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, AttachFileList& files, bool isLockPassword, int hostCountAtSameTime, bool IsNoLogging, unsigned long opt )");
+	IPMSG_FUNC_RETURN( ipmsgImpl->SendMsg( host, msg, isSecret, files, isLockPassword, hostCountAtSameTime, IsNoLogging, opt ) );
 }
 
 /**
@@ -320,7 +377,9 @@ IpMessengerAgent::SendMsg( HostListItem host, std::string msg, bool isSecret, At
 void
 IpMessengerAgent::ClearBroadcastAddress()
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::ClearBroadcastAddress()");
 	ipmsgImpl->ClearBroadcastAddress();
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -330,7 +389,9 @@ IpMessengerAgent::ClearBroadcastAddress()
 void
 IpMessengerAgent::DeleteBroadcastAddress( std::string addr )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::DeleteBroadcastAddress( std::string addr )");
 	ipmsgImpl->DeleteBroadcastAddress( addr );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -340,7 +401,9 @@ IpMessengerAgent::DeleteBroadcastAddress( std::string addr )
 void
 IpMessengerAgent::AddBroadcastAddress( std::string addr )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::AddBroadcastAddress( std::string addr )");
 	ipmsgImpl->AddBroadcastAddress( addr );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -351,7 +414,8 @@ IpMessengerAgent::AddBroadcastAddress( std::string addr )
 std::string
 IpMessengerAgent::GetInfo( HostListItem& host )
 {
-	return ipmsgImpl->GetInfo( host );
+	IPMSG_FUNC_ENTER("std::string IpMessengerAgent::GetInfo( HostListItem& host )");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetInfo( host ) );
 }
 
 /**
@@ -362,7 +426,8 @@ IpMessengerAgent::GetInfo( HostListItem& host )
 std::string
 IpMessengerAgent::GetAbsenceInfo( HostListItem& host )
 {
-	return ipmsgImpl->GetAbsenceInfo( host );
+	IPMSG_FUNC_ENTER("std::string IpMessengerAgent::GetAbsenceInfo( HostListItem& host )");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetAbsenceInfo( host ) );
 }
 
 /**
@@ -372,7 +437,8 @@ IpMessengerAgent::GetAbsenceInfo( HostListItem& host )
 std::vector<GroupItem>
 IpMessengerAgent::GetGroupList()
 {
-	return ipmsgImpl->GetGroupList();
+	IPMSG_FUNC_ENTER("std::vector<GroupItem> IpMessengerAgent::GetGroupList()");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetGroupList() );
 }
 
 /**
@@ -382,7 +448,9 @@ IpMessengerAgent::GetGroupList()
 void
 IpMessengerAgent::DeleteNotify( RecievedMessage msg )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::DeleteNotify( RecievedMessage msg )");
 	ipmsgImpl->DeleteNotify( msg );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -392,7 +460,9 @@ IpMessengerAgent::DeleteNotify( RecievedMessage msg )
 void
 IpMessengerAgent::ConfirmMessage( RecievedMessage &msg )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::ConfirmMessage( RecievedMessage &msg )");
 	ipmsgImpl->ConfirmMessage( msg );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -402,7 +472,9 @@ IpMessengerAgent::ConfirmMessage( RecievedMessage &msg )
 void
 IpMessengerAgent::AcceptConfirmNotify( SentMessage msg )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::AcceptConfirmNotify( SentMessage msg )");
 	ipmsgImpl->AcceptConfirmNotify( msg );
+	IPMSG_FUNC_EXIT;
 }
 		
 // private methods start here
@@ -414,7 +486,8 @@ IpMessengerAgent::AcceptConfirmNotify( SentMessage msg )
 int
 IpMessengerAgent::Process()
 {
-	return ipmsgImpl->Process();
+	IPMSG_FUNC_ENTER("int IpMessengerAgent::Process()");
+	IPMSG_FUNC_RETURN( ipmsgImpl->Process() );
 }
 
 /**
@@ -424,7 +497,8 @@ IpMessengerAgent::Process()
 int
 IpMessengerAgent::GetRecievedMessageCount()
 {
-	return ipmsgImpl->GetRecievedMessageCount();
+	IPMSG_FUNC_ENTER("int IpMessengerAgent::GetRecievedMessageCount()");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetRecievedMessageCount() );
 }
 
 /**
@@ -434,7 +508,8 @@ IpMessengerAgent::GetRecievedMessageCount()
 RecievedMessage
 IpMessengerAgent::PopRecievedMessage()
 {
-	return ipmsgImpl->PopRecievedMessage();
+	IPMSG_FUNC_ENTER("RecievedMessage IpMessengerAgent::PopRecievedMessage()");
+	IPMSG_FUNC_RETURN( ipmsgImpl->PopRecievedMessage() );
 }
 
 /**
@@ -444,7 +519,8 @@ IpMessengerAgent::PopRecievedMessage()
 SentMessageList *
 IpMessengerAgent::GetSentMessages()
 {
-	return ipmsgImpl->GetSentMessages();
+	IPMSG_FUNC_ENTER("SentMessageList *IpMessengerAgent::GetSentMessages()");
+	IPMSG_FUNC_RETURN( ipmsgImpl->GetSentMessages() );
 }
 
 /**
@@ -454,7 +530,8 @@ IpMessengerAgent::GetSentMessages()
 SentMessageList
 IpMessengerAgent::CloneSentMessages() const
 {
-	return ipmsgImpl->CloneSentMessages();
+	IPMSG_FUNC_ENTER("SentMessageList IpMessengerAgent::CloneSentMessages() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->CloneSentMessages() );
 }
 
 /**
@@ -464,7 +541,8 @@ IpMessengerAgent::CloneSentMessages() const
 std::string
 IpMessengerAgent::LoginName() const
 {
-	return ipmsgImpl->LoginName();
+	IPMSG_FUNC_ENTER("std::string IpMessengerAgent::LoginName() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->LoginName() );
 }
 
 /**
@@ -474,7 +552,8 @@ IpMessengerAgent::LoginName() const
 std::string
 IpMessengerAgent::HostName() const
 {
-	return ipmsgImpl->HostName();
+	IPMSG_FUNC_ENTER("std::string IpMessengerAgent::HostName() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->HostName() );
 }
 
 /**
@@ -484,7 +563,8 @@ IpMessengerAgent::HostName() const
 int
 IpMessengerAgent::DefaultPortNo() const
 {
-	return ipmsgImpl->DefaultPortNo();
+	IPMSG_FUNC_ENTER("int IpMessengerAgent::DefaultPortNo() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->DefaultPortNo() );
 }
 
 /**
@@ -494,7 +574,9 @@ IpMessengerAgent::DefaultPortNo() const
 void
 IpMessengerAgent::setDefaultPortNo( const int defaultPortNo )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::setDefaultPortNo( const int defaultPortNo )");
 	ipmsgImpl->setDefaultPortNo( defaultPortNo );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -504,7 +586,8 @@ IpMessengerAgent::setDefaultPortNo( const int defaultPortNo )
 bool
 IpMessengerAgent::IsDialup() const
 {
-	return ipmsgImpl->IsDialup();
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::IsDialup() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->IsDialup() );
 }
 
 /**
@@ -514,7 +597,9 @@ IpMessengerAgent::IsDialup() const
 void
 IpMessengerAgent::setIsDialup( const bool isDialup )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::setIsDialup( const bool isDialup )");
 	ipmsgImpl->setIsDialup( isDialup );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -524,7 +609,8 @@ IpMessengerAgent::setIsDialup( const bool isDialup )
 bool
 IpMessengerAgent::AbortDownloadAtFileChanged() const
 {
-	return ipmsgImpl->AbortDownloadAtFileChanged();
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::AbortDownloadAtFileChanged() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->AbortDownloadAtFileChanged() );
 }
 
 /**
@@ -534,7 +620,9 @@ IpMessengerAgent::AbortDownloadAtFileChanged() const
 void
 IpMessengerAgent::setAbortDownloadAtFileChanged( const bool isAbort )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::setAbortDownloadAtFileChanged( const bool isAbort )");
 	ipmsgImpl->setAbortDownloadAtFileChanged( isAbort );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -544,7 +632,8 @@ IpMessengerAgent::setAbortDownloadAtFileChanged( const bool isAbort )
 bool
 IpMessengerAgent::SaveSentMessage() const
 {
-	return ipmsgImpl->SaveSentMessage();
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::SaveSentMessage() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->SaveSentMessage() );
 }
 
 /**
@@ -554,7 +643,9 @@ IpMessengerAgent::SaveSentMessage() const
 void
 IpMessengerAgent::setSaveSentMessage( const bool isSave )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::setSaveSentMessage( const bool isSave )");
 	ipmsgImpl->setSaveSentMessage( isSave );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -564,7 +655,8 @@ IpMessengerAgent::setSaveSentMessage( const bool isSave )
 bool
 IpMessengerAgent::SaveRecievedMessage() const
 {
-	return ipmsgImpl->SaveRecievedMessage();
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::SaveRecievedMessage() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->SaveRecievedMessage() );
 }
 
 /**
@@ -574,25 +666,88 @@ IpMessengerAgent::SaveRecievedMessage() const
 void
 IpMessengerAgent::setSaveRecievedMessage( const bool isSave )
 {
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::setSaveRecievedMessage( const bool isSave )");
 	ipmsgImpl->setSaveRecievedMessage( isSave );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
  * Ź沽˼ԤåʤɤΥե饰Υå
  * @retval Ź沽˼ԤåʤɤΥե饰
  */
-bool IpMessengerAgent::NoSendMessageOnEncryptionFailed() const
+bool
+IpMessengerAgent::NoSendMessageOnEncryptionFailed() const
 {
-	return ipmsgImpl->NoSendMessageOnEncryptionFailed();
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::NoSendMessageOnEncryptionFailed() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->NoSendMessageOnEncryptionFailed() );
 }
 
 /**
  * Ź沽˼ԤåʤɤΥե饰Υå
- * @param isSave Ź沽˼ԤåʤɤΥե饰
+ * @param isNoSend Ź沽˼ԤåʤɤΥե饰
  */
-void IpMessengerAgent::setNoSendMessageOnEncryptionFailed( const bool isNoSend )
+void
+IpMessengerAgent::setNoSendMessageOnEncryptionFailed( const bool isNoSend )
 {
-	return ipmsgImpl->setNoSendMessageOnEncryptionFailed( isNoSend );
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::setNoSendMessageOnEncryptionFailed( const bool isNoSend )");
+	IPMSG_FUNC_RETURN( ipmsgImpl->setNoSendMessageOnEncryptionFailed( isNoSend ) );
+}
+
+/**
+ * IPv6ȤɤΥե饰Υå
+ * @retval IPv6ȤɤΥե饰
+ */
+bool
+IpMessengerAgent::UseIPv6() const
+{
+	IPMSG_FUNC_ENTER("bool IpMessengerAgent::UseIPv6() const");
+	IPMSG_FUNC_RETURN( ipmsgImpl->UseIPv6() );
+}
+
+/**
+ * IPv6ȤɤΥե饰Υå
+ * @param useIPv6 IPv6ȤɤΥե饰
+ */
+void
+IpMessengerAgent::setUseIPv6( const bool useIPv6 )
+{
+	IPMSG_FUNC_ENTER("void IpMessengerAgent::setUseIPv6( const bool useIPv6 )");
+#ifdef ENABLE_IPV6
+	ipmsgImpl->setUseIPv6( useIPv6 );
+#else
+	ipmsgImpl->setUseIPv6( false );
+#endif
+	IPMSG_FUNC_EXIT;
+}
+
+/**
+ * IPv6򥵥ݡȤƤ뤫ӥɥץ롣
+ * @retval true:IPv6򥵥ݡ
+ * @retval false:IPv6򥵥ݡȤƤʤ
+ */
+bool
+IpMessengerAgent::isSupportIPv6()
+{
+#ifdef ENABLE_IPV6
+	return true;
+#else
+	return false;
+#endif
+}
+
+/**
+ * IPv4򥵥ݡȤƤ뤫ӥɥץ롣
+ * @retval true:IPv4򥵥ݡ
+ * @retval false:IPv4򥵥ݡȤƤʤ
+ */
+bool
+IpMessengerAgent::isSupportIPv4()
+{
+#ifdef ENABLE_IPV4
+	return true;
+#else
+	return false;
+#endif
 }
 
 /**
@@ -602,7 +757,8 @@ void IpMessengerAgent::setNoSendMessageOnEncryptionFailed( const bool isNoSend )
 long double
 DownloadInfo::getSpeed()
 {
-	return Time() == 0 ? (long double)0 : ( ( long double )Size() / ( long double )Time() );
+	IPMSG_FUNC_ENTER("long double DownloadInfo::getSpeed()");
+	IPMSG_FUNC_RETURN( Time() == 0 ? (long double)0 : ( ( long double )Size() / ( long double )Time() ) );
 }
 
 /**
@@ -612,7 +768,8 @@ DownloadInfo::getSpeed()
 std::string
 DownloadInfo::getSpeedString()
 {
-	return DownloadInfo::getUnitSizeString( ( long long )getSpeed() ) + "/sec";
+	IPMSG_FUNC_ENTER("std::string DownloadInfo::getSpeedString()");
+	IPMSG_FUNC_RETURN( DownloadInfo::getUnitSizeString( ( long long )getSpeed() ) + "/sec" );
 }
 
 /**
@@ -622,7 +779,8 @@ DownloadInfo::getSpeedString()
 std::string
 DownloadInfo::getSizeString()
 {
-	return DownloadInfo::getUnitSizeString( Size() );
+	IPMSG_FUNC_ENTER("std::string DownloadInfo::getSizeString()");
+	IPMSG_FUNC_RETURN( DownloadInfo::getUnitSizeString( Size() ) );
 }
 
 #define IPMSG_SIZE_B	(long double)(1)
@@ -638,23 +796,37 @@ DownloadInfo::getSizeString()
 std::string
 DownloadInfo::getUnitSizeString( long long size )
 {
+	IPMSG_FUNC_ENTER("std::string DownloadInfo::getUnitSizeString( long long size )");
 	long double dsize = (long double)size;
 	char buf[100];
 	if ( dsize >= IPMSG_SIZE_TB ) {
 		snprintf( buf, sizeof( buf ), "%.2Lf TB", (dsize / IPMSG_SIZE_TB) );
-		return buf;
+		IPMSG_FUNC_RETURN( buf );
 	} else if ( dsize >= IPMSG_SIZE_GB ) {
 		snprintf( buf, sizeof( buf ), "%.2Lf GB", (dsize / IPMSG_SIZE_GB) );
-		return buf;
+		IPMSG_FUNC_RETURN( buf );
 	} else if ( dsize >= IPMSG_SIZE_MB ) {
 		snprintf( buf, sizeof( buf ), "%.2Lf MB", (dsize / IPMSG_SIZE_MB) );
-		return buf;
+		IPMSG_FUNC_RETURN( buf );
 	} else if ( dsize >= IPMSG_SIZE_KB ) {
 		snprintf( buf, sizeof( buf ), "%.2Lf KB", (dsize / IPMSG_SIZE_KB) );
-		return buf;
+		IPMSG_FUNC_RETURN( buf );
 	}
 	snprintf( buf, sizeof( buf ), "%lld B", size );
-	return buf;
+	IPMSG_FUNC_RETURN( buf );
+}
+
+/**
+ * ǥХ̾ꤷϡɥɥ쥹롣
+ * @param val ǥХ̾
+ */
+void
+NetworkInterface::setDeviceName( const std::string val )
+{
+	IPMSG_FUNC_ENTER("void NetworkInterface::setDeviceName( const std::string val )");
+	_DeviceName = val;
+	_HardwareAddress = getNetworkInterfaceMacAddress( val );
+	IPMSG_FUNC_EXIT;
 }
 
 /**
@@ -664,10 +836,12 @@ DownloadInfo::getUnitSizeString( long long size )
 void
 NetworkInterface::setIpAddress( const std::string val )
 {
+	IPMSG_FUNC_ENTER("void NetworkInterface::setIpAddress( const std::string val )");
 	_IpAddress = val;
-	inet_pton( AF_INET, val.c_str(), (void *)&_NativeIpAddress );
 	recalc();
+	IPMSG_FUNC_EXIT;
 }
+
 /**
  * ͥåȥޥꤷͥåȥɥ쥹֥ɥ㥹ȥɥ쥹Ʒ׻롣
  * @param val ͥåȥޥʸ
@@ -675,45 +849,21 @@ NetworkInterface::setIpAddress( const std::string val )
 void
 NetworkInterface::setNetMask( const std::string val )
 {
+	IPMSG_FUNC_ENTER("void NetworkInterface::setNetMask( const std::string val )");
 	_NetMask = val;
-	inet_pton( AF_INET, val.c_str(), (void *)&_NativeNetMask );
-	recalc();
-}
-/**
- * IPɥ쥹(ͥƥ)ꤷͥåȥɥ쥹֥ɥ㥹ȥɥ쥹Ʒ׻롣
- * @param val IPɥ쥹
- */
-void
-NetworkInterface::setNativeIpAddress( const struct in_addr val )
-{
-	_NativeIpAddress = val;
-	char ipAddrBuf[IP_ADDR_MAX_SIZE];
-	_IpAddress = inet_ntop( AF_INET, &val, ipAddrBuf, sizeof( ipAddrBuf ) );
-	recalc();
-}
-/**
- * ͥåȥޥ(ͥƥ)ꤷͥåȥɥ쥹֥ɥ㥹ȥɥ쥹Ʒ׻롣
- * @param val ͥåȥޥ
- */
-void
-NetworkInterface::setNativeNetMask( const struct in_addr val )
-{
-	_NativeNetMask = val;
-	char netMaskBuf[IP_ADDR_MAX_SIZE];
-	_NetMask = inet_ntop( AF_INET, &val, netMaskBuf, sizeof( netMaskBuf ) );
 	recalc();
+	IPMSG_FUNC_EXIT;
 }
+
 /**
  * ͥåȥɥ쥹֥ɥ㥹ȥɥ쥹׻롣
  */
 void
 NetworkInterface::recalc()
 {
-	char buf[IP_ADDR_MAX_SIZE];
-	_NativeNetworkAddress.s_addr = _NativeIpAddress.s_addr & _NativeNetMask.s_addr;
-	_NetworkAddress = inet_ntop( AF_INET, &_NativeNetworkAddress, buf, sizeof( buf ) );
-
-	_NativeBroadcastAddress = GetBroadcastAddress( _NativeNetworkAddress, _NativeNetMask );
-	_BroadcastAddress = inet_ntop( AF_INET, &_NativeBroadcastAddress, buf, sizeof( buf ) );
+	IPMSG_FUNC_ENTER("void NetworkInterface::recalc()");
+	_NetworkAddress = getNetworkAddress( _AddressFamily, _IpAddress, _NetMask );
+	_BroadcastAddress = getBroadcastAddress( _AddressFamily, _NetworkAddress, _NetMask );
+	IPMSG_FUNC_EXIT;
 }
 //end of source
