@@ -91,7 +91,8 @@ class Packet{
 		IPMSG_PROPERTY( std::string, HostName );
 		IPMSG_PROPERTY( std::string, UserName );
 		IPMSG_PROPERTY( std::string, Option );
-		IPMSG_PROPERTY( struct sockaddr_in, Addr );
+		IPMSG_PROPERTY( struct sockaddr_storage, Addr );
+		IPMSG_PROPERTY( int, UdpSocket );
 		IPMSG_PROPERTY( int, TcpSocket );
 };
 
@@ -100,24 +101,21 @@ class Packet{
  **/
 class NetworkInterface {
 	public:
-		IPMSG_PROPERTY( std::string, DeviceName );
+		IPMSG_READONLY_PROPERTY( std::string, DeviceName );
+		IPMSG_PROPERTY( int, AddressFamily );
 		IPMSG_READONLY_PROPERTY( std::string, IpAddress );
+		IPMSG_READONLY_PROPERTY( std::string, HardwareAddress );
 		IPMSG_READONLY_PROPERTY( std::string, NetMask );
 		IPMSG_READONLY_PROPERTY( std::string, NetworkAddress );
 		IPMSG_READONLY_PROPERTY( std::string, BroadcastAddress );
-		IPMSG_READONLY_PROPERTY( struct in_addr, NativeIpAddress );
-		IPMSG_READONLY_PROPERTY( struct in_addr, NativeNetMask );
-		IPMSG_READONLY_PROPERTY( struct in_addr, NativeNetworkAddress );
-		IPMSG_READONLY_PROPERTY( struct in_addr, NativeBroadcastAddress );
 		IPMSG_PROPERTY( int, PortNo );
-		NetworkInterface( std::string deviceName ):_DeviceName( deviceName ){};
-		NetworkInterface(){};
+		NetworkInterface( std::string deviceName ):_AddressFamily( AF_INET ){ setDeviceName( deviceName ); };
+		NetworkInterface( int addressFamily, std::string deviceName ):_AddressFamily( addressFamily ){ setDeviceName( deviceName ); };
+		NetworkInterface():_AddressFamily( AF_INET ){};
 	public:
+		void setDeviceName( const std::string val );
 		void setIpAddress( const std::string val );
 		void setNetMask( const std::string val );
-		void setNativeIpAddress( const struct in_addr val );
-		void setNativeNetMask( const struct in_addr val );
-	private:
 		void recalc();
 };
 
@@ -140,7 +138,10 @@ class HostListItem{
 		IPMSG_PROPERTY( std::string, UserName );
 		IPMSG_PROPERTY( std::string, HostName );
 		IPMSG_PROPERTY( unsigned long, CommandNo );
-		IPMSG_PROPERTY( std::string, IpAddress );
+		IPMSG_PROPERTY( int, AddressFamily );
+		std::string IpAddress() const;
+		void setIpAddress( const std::string val );
+		IPMSG_READONLY_PROPERTY( std::string, HardwareAddress );
 		IPMSG_PROPERTY( std::string, Nickname );
 		IPMSG_PROPERTY( std::string, GroupName );
 		IPMSG_PROPERTY( std::string, EncodingName );
@@ -154,9 +155,13 @@ class HostListItem{
 		bool IsEncryptSupport() const;
 		bool IsAbsence() const;
 		bool Equals( const HostListItem& item ) const;
+		bool EqualsHardwareAddress( const HostListItem& item ) const;
 		int Compare( const HostListItem& item ) const;
+		int CompareHardwareAddress( const HostListItem& item ) const;
 		void QueryVersionInfo();
 		void QueryAbsenceInfo();
+	private:
+		std::string _IpAddress;
 };
 
 /**
@@ -174,6 +179,7 @@ class HostListComparator{
 		 * @retval +n:host2礭
 		 */
 		virtual int compare( std::vector<HostListItem>::iterator host1, std::vector<HostListItem>::iterator host2 )=0;
+		virtual ~HostListComparator(){};
 };
 
 /**
@@ -203,17 +209,17 @@ class HostList{
 		IPMSG_PROPERTY( time_t, AskStartTime );
 		IPMSG_PROPERTY( time_t, PrevTry );
 		IPMSG_PROPERTY( int, RetryCount );
-		void AddHost( const HostListItem& host );
+		int AddHost( const HostListItem& host, bool isPermitSameHardwareAddress=true );
 		void Delete( std::vector<HostListItem>::iterator &it );
 		void DeleteHostByAddress( std::string addr );
 		std::vector<HostListItem>::iterator FindHostByAddress( std::string addr );
-		std::vector<HostListItem>::iterator FindHostByHostName( std::string hostName );
+		std::vector<HostListItem>::iterator FindHostByHostName( std::string hostName, int addressFamily );
 		static HostListItem CreateHostListItemFromPacket( const Packet& packet );
 		std::vector<HostListItem>::iterator begin();
 		std::vector<HostListItem>::iterator end();
 		int size() const;
 		void clear();
-		std::string ToString( int start, const struct sockaddr_in *addr );
+		std::string ToString( int start, const struct sockaddr_storage *addr );
 		void sort( HostListComparator *comparator );
 		HostList();
 		HostList( const HostList& other );
@@ -237,6 +243,7 @@ class FileNameConverter {
 	public:
 		virtual std::string ConvertNetworkToLocal( std::string original_file_name ) = 0;
 		virtual std::string ConvertLocalToNetwork( std::string original_file_name ) = 0;
+		virtual ~FileNameConverter(){};
 };
 
 /**
@@ -384,7 +391,7 @@ class RecievedMessageList {
  **/
 class SentMessage{
 	public:
-		IPMSG_PROPERTY( struct sockaddr_in, To );
+		IPMSG_PROPERTY( struct sockaddr_storage, To );
 		IPMSG_PROPERTY( HostListItem, Host );
 		IPMSG_PROPERTY( unsigned long, PacketNo );
 		IPMSG_PROPERTY( std::string, Message );
@@ -484,6 +491,16 @@ class IpMessengerEvent {
 		 * @param msg å
 		 */
 		virtual void SendAfter( SentMessage& msg )=0;
+		/**
+		 * åŹ沽Υ٥ȡ
+		 */
+		virtual void NotifySendEncryptionFail( HostListItem& host )=0;
+		/**
+		 * åŹ沽ԥ٥ȡ
+		 * @retval true:Ź沽
+		 * @retval false:Ԥ
+		 */
+		virtual bool IsSendContinueOnEncryptionFail( HostListItem& host )=0;
 		/**
 		 * åȥ饤顼٥
 		 * @param msg å
@@ -583,7 +600,7 @@ class IpMessengerAgent {
 		/**
 		 * NICξ
 		 **/
-		static void GetNetworkInterfaceInfo( std::vector<NetworkInterface>& nics );
+		static void GetNetworkInterfaceInfo( std::vector<NetworkInterface>& nics, bool useIPv6 = false );
 
 		/**
 		 * ֥ɥ㥹ȥɥ쥹ΥꥹȤ򥯥ꥢ
@@ -805,6 +822,18 @@ class IpMessengerAgent {
 		 **/
 		bool NoSendMessageOnEncryptionFailed() const;
 		void setNoSendMessageOnEncryptionFailed( const bool isNoSend );
+
+		/**
+		 * IPv6Ѥ롣
+		 **/
+		bool UseIPv6() const;
+		void setUseIPv6( const bool useIPv6 );
+
+		/**
+		 * APIΥݡȾĴ٤롣
+		 **/
+		bool isSupportIPv6();
+		bool isSupportIPv4();
 	private:
 		IpMessengerAgent();
 		~IpMessengerAgent();
